@@ -11,10 +11,9 @@
 
 #![recursion_limit = "1024"] // Prevent select! macro blowing up
 
-///! forks of logterm customise the files in src/custom
 #[path = "../custom/mod.rs"]
 pub mod custom;
-use self::custom::app::{set_main_view, App, DashViewMain};
+use self::custom::app::{OPT, App, DashViewMain, set_main_view};
 use self::custom::ui::draw_dashboard;
 
 #[macro_use]
@@ -56,9 +55,13 @@ enum Event<I> {
 use tokio_stream::StreamExt;
 use tokio::sync::mpsc;
 
-// RUSTFLAGS="-A unused" cargo run --bin logtail-crossterm --features="crossterm" /var/log/auth.log /var/log/dmesg
 #[tokio::main]
 pub async fn main() -> Result<(), Box<dyn Error>> {
+	let (opt_tick_rate, checkpoint_interval, opt_debug_window) = {
+		let opt = OPT.lock().unwrap();
+		(opt.tick_rate, opt.checkpoint_interval, opt.debug_window)
+	};
+
 	env_logger::init();
 	info!("Started");
 
@@ -69,11 +72,12 @@ pub async fn main() -> Result<(), Box<dyn Error>> {
 
 	// Terminal initialization
 	enable_raw_mode()?;
+
 	let mut stdout = stdout();
 	execute!(stdout, EnterAlternateScreen, EnableMouseCapture)?;
 	let backend = CrosstermBackend::new(stdout);
 	let mut terminal = Terminal::new(backend)?;
-	let mut rx = initialise_events(app.opt.tick_rate);
+	let mut rx = initialise_events(opt_tick_rate);
 	terminal.clear()?;
 
 	// Use futures of async functions to handle events
@@ -91,7 +95,7 @@ pub async fn main() -> Result<(), Box<dyn Error>> {
 			next_update += Duration::from_secs(1);
 		}
 
-		let logfiles_future = app.logfiles.next().fuse();
+		let logfiles_future = app.logfiles_manager.linemux_files.next().fuse();
 		let events_future = rx.recv().fuse();
 		pin_mut!(logfiles_future, events_future);
 
@@ -115,12 +119,18 @@ pub async fn main() -> Result<(), Box<dyn Error>> {
 							break Ok(());
 						},
 						KeyCode::Enter => {
-							if app.dash_state.main_view == DashViewMain::DashNode {
-								app.preserve_node_selection();
-								set_main_view(DashViewMain::DashSummary, &mut app);
-							} else if app.dash_state.main_view == DashViewMain::DashSummary {
-								app.preserve_node_selection();
-								set_main_view(DashViewMain::DashNode, &mut app);
+							if app.dash_state.main_view == DashViewMain::DashHelp {
+								set_main_view(app.dash_state.previous_main_view, &mut app);
+							} else {
+								if app.logfiles_manager.logfiles_added.len() > 0 {
+									if app.dash_state.main_view == DashViewMain::DashNode {
+										app.preserve_node_selection();
+										set_main_view(DashViewMain::DashSummary, &mut app);
+									} else if app.dash_state.main_view == DashViewMain::DashSummary {
+										app.preserve_node_selection();
+										set_main_view(DashViewMain::DashNode, &mut app);
+									}
+								}
 							}
 						}
 
@@ -142,8 +152,10 @@ pub async fn main() -> Result<(), Box<dyn Error>> {
 						KeyCode::Char('?') => set_main_view(DashViewMain::DashHelp, &mut app),
 						KeyCode::Char('n')|
 						KeyCode::Char('N') => {
-							app.preserve_node_selection();
-							set_main_view(DashViewMain::DashNode, &mut app);
+							if app.logfiles_manager.logfiles_added.len() > 0 {
+								app.preserve_node_selection();
+								set_main_view(DashViewMain::DashNode, &mut app);
+							}
 						},
 
 						KeyCode::Char('+')|
@@ -159,6 +171,9 @@ pub async fn main() -> Result<(), Box<dyn Error>> {
 						KeyCode::Char('m')|
 						KeyCode::Char('M') => app.bump_mmm_ui_mode(),
 
+						KeyCode::Char('r')|
+						KeyCode::Char('R') => app.scan_glob_paths(false, false).await,
+
 						KeyCode::Char('t') => app.top_timeline_next(),
 						KeyCode::Char('T') => app.top_timeline_previous(),
 
@@ -169,7 +184,7 @@ pub async fn main() -> Result<(), Box<dyn Error>> {
 						KeyCode::Left => app.change_focus_previous(),
 
 						KeyCode::Char('g') => {
-							if app.opt.debug_window { set_main_view(DashViewMain::DashDebug, &mut app); }
+							if opt_debug_window { set_main_view(DashViewMain::DashDebug, &mut app); }
 						},
 						_ => {}
 					};
@@ -178,14 +193,14 @@ pub async fn main() -> Result<(), Box<dyn Error>> {
 
 				Some(Event::Tick) => {
 					app.update_timelines(&Utc::now());
+					app.scan_glob_paths(true, true).await;
 				// draw_dashboard(&mut f, &dash_state, &mut monitors).unwrap();
 				// draw_dashboard(f, &dash_state, &mut monitors)?;
 				}
 
 				None => {},
 			}
-			},
-
+		},
 			line = logfiles_future => {
 			match line {
 				Some(Ok(line)) => {
@@ -194,9 +209,10 @@ pub async fn main() -> Result<(), Box<dyn Error>> {
 					let source = String::from(source_str);
 					// app.dash_state._debug_window(format!("{}: {}", source, line.line()).as_str());
 
+					let mut checkpoint_result: Result<String, std::io::Error> = Ok("".to_string());
 					match app.get_monitor_for_file_path(&source) {
 						Some(monitor) => {
-							monitor.append_to_content(line.line())?;
+							checkpoint_result = monitor.append_to_content(line.line(), checkpoint_interval);
 							if monitor.is_debug_dashboard_log {
 								app.dash_state._debug_window(line.line());
 							} else if app.dash_state.main_view == DashViewMain::DashSummary {
@@ -207,6 +223,16 @@ pub async fn main() -> Result<(), Box<dyn Error>> {
 							app.dash_state._debug_window(format!("NO MONITOR FOR: {}", source).as_str());
 						},
 					}
+					match checkpoint_result {
+						Ok(message) => {
+							if message.len() > 0 {
+								app.dash_state.vdash_status.message(&message, None);
+							}
+						},
+						Err(e) => {
+							app.dash_state.vdash_status.message(&e.to_string(), None);
+						}
+					}
 				},
 				Some(Err(e)) => {
 					app.dash_state._debug_window(format!("logfile error: {:#?}", e).as_str());
@@ -216,11 +242,12 @@ pub async fn main() -> Result<(), Box<dyn Error>> {
 					app.dash_state._debug_window(format!("logfile error: None").as_str());
 					()
 				}
-		}
-			},
+			}
+		},
 		}
 	}
 }
+
 type Rx = tokio::sync::mpsc::UnboundedReceiver<Event<crossterm::event::KeyEvent>>;
 
 fn initialise_events(tick_rate: u64) -> Rx {
@@ -235,7 +262,7 @@ fn initialise_events(tick_rate: u64) -> Rx {
 				if let CEvent::Key(key) = event::read().unwrap() {
 					match tx.send(Event::Input(key)) {
 						Ok(()) => {},
-						Err(e) => println!("send error: {}", e),
+						Err(e) => eprintln!("send error: {}", e),
 
 					}
 				}
@@ -243,7 +270,7 @@ fn initialise_events(tick_rate: u64) -> Rx {
 			if last_tick.elapsed() >= tick_rate {
 				match tx.send(Event::Tick) {
 					Ok(()) => last_tick = Instant::now(),
-					Err(e) => println!("send error: {}", e),
+					Err(e) => eprintln!("send error: {}", e),
 
 				}
 			}
@@ -251,10 +278,11 @@ fn initialise_events(tick_rate: u64) -> Rx {
 			if last_tick.elapsed() >= tick_rate {
 				match tx.send(Event::Tick) {
 					Ok(()) => last_tick = Instant::now(),
-					Err(e) => println!("send error: {}", e),
+					Err(e) => eprintln!("send error: {}", e),
 				}
 			}
 		}
 	});
 	rx
 }
+
