@@ -1,22 +1,26 @@
 ///! Application logic
-///!
-///! Edit src/custom/app.rs to create a customised fork of logtail-dash
-use linemux::MuxedLines;
-use std::collections::HashMap;
+//
+// TODO consider colouring logfiles using regex's from https://github.com/bensadeh/tailspin
 
+use std::collections::HashMap;
 use std::fs::File;
 use std::io::{Error, ErrorKind, Write};
-use std::path::{Path, PathBuf};
+use std::path::Path;
+
+use serde::{Serialize, Deserialize};
 use chrono::{DateTime, Utc, Duration};
 use structopt::StructOpt;
 use tempfile::NamedTempFile;
 
-use crate::custom::timelines::{MinMeanMax, get_duration_text};
-use crate::custom::app_timelines::{AppTimelines, TIMESCALES, APP_TIMELINES};
-use crate::custom::app_timelines::{STORAGE_COST_TIMELINE_KEY, EARNINGS_TIMELINE_KEY, PUTS_TIMELINE_KEY, GETS_TIMELINE_KEY, CONNECTIONS_TIMELINE_KEY, RAM_TIMELINE_KEY, ERRORS_TIMELINE_KEY};
-use crate::custom::opt::{Opt, MIN_TIMELINE_STEPS};
 use crate::shared::util::StatefulList;
 
+use super::timelines::{MinMeanMax, get_duration_text};
+use super::app_timelines::{AppTimelines, TIMESCALES, APP_TIMELINES};
+use super::app_timelines::{STORAGE_COST_TIMELINE_KEY, EARNINGS_TIMELINE_KEY, PUTS_TIMELINE_KEY, GETS_TIMELINE_KEY, CONNECTIONS_TIMELINE_KEY, RAM_TIMELINE_KEY, ERRORS_TIMELINE_KEY};
+use super::opt::{Opt, MIN_TIMELINE_STEPS};
+use super::logfiles_manager::LogfilesManager;
+use super::logfile_checkpoints::save_checkpoint;
+
 pub const SAFENODE_BINARY_NAME: &str = "safenode";
 pub static SUMMARY_WINDOW_NAME: &str = "Summary of Monitored Nodes";
 pub static HELP_WINDOW_NAME: &str = "Help";
@@ -53,25 +57,43 @@ pub unsafe fn debug_log(message: &str) {
 	};
 }
 
+lazy_static::lazy_static! {
+	pub static ref OPT: Mutex<Opt> =
+		Mutex::<Opt>::new(Opt::from_args());
+}
+
 pub struct App {
-	pub opt: Opt,
 	pub dash_state: DashState,
 	pub monitors: HashMap<String, LogMonitor>,
 	pub logfile_with_focus: String,
-	pub logfiles: MuxedLines,
+
+	pub logfiles_manager: LogfilesManager,
+	pub next_glob_scan: Option<DateTime<Utc>>,
 }
 
 impl App {
 	pub async fn new() -> Result<App, std::io::Error> {
-		let mut opt = Opt::from_args();
+		let (opt_files, opt_globpaths, opt_debug_window, opt_timeline_steps) = {
+			let opt = OPT.lock().unwrap();
+			(opt.files.clone(), opt.glob_paths.clone(), opt.debug_window, opt.timeline_steps)
+		};
 
-		if opt.files.is_empty() {
-			println!("{}: no logfile(s) specified.", Opt::clap().get_name());
+		let mut app = App {
+			dash_state: DashState::new(),
+			monitors: HashMap::new(),
+			logfile_with_focus: String::new(),
+
+			logfiles_manager: LogfilesManager::new(opt_globpaths.clone()),
+			next_glob_scan: None,
+		};
+
+		if opt_files.is_empty() && opt_globpaths.is_empty() {
+			eprintln!("{}: no logfile(s) or 'glob' paths provided.", Opt::clap().get_name());
 			return exit_with_usage("missing logfiles");
 		}
 
-		if opt.timeline_steps < MIN_TIMELINE_STEPS {
-			println!(
+		if opt_timeline_steps < MIN_TIMELINE_STEPS {
+			eprintln!(
 				"Timeline steps number is too small, minimum is {}",
 				MIN_TIMELINE_STEPS
 			);
@@ -79,106 +101,85 @@ impl App {
 		}
 
 		let mut dash_state = DashState::new();
-		dash_state.debug_window = opt.debug_window;
-		if opt.debug_dashboard {
+		dash_state.debug_window = opt_debug_window;
+		if opt_debug_window {
 			dash_state.main_view = DashViewMain::DashDebug;
 		}
 
-		let mut monitors: HashMap<String, LogMonitor> = HashMap::new();
-		let mut logfiles = MuxedLines::new()?;
-		let mut debug_logfile_name = String::new();
+		let mut files_to_load = opt_files.clone();
 
-		let mut debug_logfile: Option<tempfile::NamedTempFile> = if opt.debug_window {
-			opt.files = opt.files[0..1].to_vec();
-			let named_file = NamedTempFile::new()?;
-			let path = named_file.path();
+		if opt_debug_window {
+			if opt_files.len() == 0 {
+				eprint!("For debugging with --debug-window you must specify a logfile path.");
+				return exit_with_usage("missing logfile");
+			}
+
+			// For debug: only use first logfile, plus one for debug messages
+			files_to_load = opt_files[0..1].to_vec();
+			let debug_file = NamedTempFile::new()?;
+			let path = debug_file.path();
 			let path_str = path
 				.to_str()
 				.ok_or_else(|| Error::new(ErrorKind::Other, "invalid path"))?;
-			opt.files.push(String::from(path_str));
-			debug_logfile_name = String::from(path_str);
-			Some(named_file)
-		} else {
-			None
-		};
+			files_to_load.push(String::from(path_str));
+			*DEBUG_LOGFILE.lock().unwrap() = Some(debug_file);
+		}
+
+		if files_to_load.len() > 0 {
+			app.logfiles_manager.monitor_multi_paths(files_to_load, &mut app.monitors, &mut app.dash_state, false).await;
+		}
 
-		println!("Loading {} files...", opt.files.len());
-		let mut first_logfile = String::new();
-		for f in &opt.files {
-			println!("file: {}", f);
-			if first_logfile.is_empty() {
-				first_logfile = f.to_string();
-			}
-			let mut monitor = LogMonitor::new(&opt, f.to_string(), opt.lines_max);
-			if opt.debug_window && monitor.index == 0 {
-				if let Some(named_file) = debug_logfile {
-					*DEBUG_LOGFILE.lock().unwrap() = Some(named_file);
-					debug_logfile = None;
-				}
-			}
-			if opt.ignore_existing {
-				dash_state.logfile_names.push(f.to_string());
-				monitors.insert(f.to_string(), monitor);
-			} else {
-				match monitor.load_logfile(&mut dash_state) {
-					Ok(()) => {
-						dash_state.logfile_names.push(f.to_string());
-						monitors.insert(f.to_string(), monitor);
-					}
-					Err(e) => {
-						println!("...failed: {}", e);
-						return Err(e);
-					}
-				}
-			}
+		app.scan_glob_paths(false, false).await;
 
-			match logfiles.add_file(&f).await {
-				Ok(_) => (),
-				Err(e) => {
-					println!("ERROR: {}", e);
-					println!(
-						"Note: it is ok for the file not to exist, but the file's parent directory must exist."
-					);
-					return Err(e);
-				}
-			}
+		if app.logfiles_manager.logfiles_added.len() > 0 {
+			app.logfile_with_focus = app.logfiles_manager.logfiles_added[0].clone();	// Save to give focus
+		} else {
+			app.dash_state.vdash_status.message(&"No files to monitor, please start a node and try again.".to_string(), None);
+			return exit_with_usage("no files to monitor.");
 		}
 
-		let activate_debug_dashboard = opt.debug_dashboard;
-		let mut app = App {
-			opt,
-			dash_state,
-			monitors,
-			logfile_with_focus: first_logfile.clone(),
-			logfiles,
-		};
 		app.update_timelines(&Utc::now());
 		app.update_summary_window();
 
-		if !first_logfile.is_empty() {
-			app.dash_state.dash_node_focus = first_logfile.clone();
+		if !app.logfile_with_focus.is_empty() {
+			app.dash_state.dash_node_focus = app.logfile_with_focus.clone();
 		}
 
-		if activate_debug_dashboard {
-			app.set_logfile_with_focus(debug_logfile_name);
-		} else {
-			app.set_logfile_with_focus(first_logfile);
-		}
+		app.set_logfile_with_focus(app.logfile_with_focus.clone());
+		app.dash_state.vdash_status.disable_to_console();
 		Ok(app)
 	}
 
+	pub async fn scan_glob_paths(&mut self, timed: bool, disable_status: bool) {
+		if self.logfiles_manager.globpaths.len() == 0 { return; }
+		let opt_globs_scan = OPT.lock().unwrap().glob_scan;
+
+		let mut do_scan = !timed;
+		if timed && opt_globs_scan > 0 {
+			let current_time = Utc::now();
+			if let Some(next_glob_scan) = self.next_glob_scan {
+				if current_time > next_glob_scan {
+					self.next_glob_scan = Some(current_time + Duration::seconds(opt_globs_scan));
+					do_scan = true;
+				}
+			} else {
+				self.next_glob_scan = Some(current_time + Duration::seconds(opt_globs_scan));
+				do_scan = true;
+			}
+		}
+
+		if do_scan {
+			let opt_glob_paths = OPT.lock().unwrap().glob_paths.clone();
+			self.logfiles_manager.scan_multi_globpaths(opt_glob_paths, &mut self.monitors, &mut self.dash_state, disable_status).await;
+		}
+	}
+
 	pub fn update_timelines(&mut self, now: &DateTime<Utc>) {
 		for (_monitor_file, monitor) in self.monitors.iter_mut() {
 			monitor.metrics.update_timelines(now);
 		}
 	}
 
-	// pub fn update_chunk_store_stats(&mut self) {
-	// 	for (_monitor_file, monitor) in self.monitors.iter_mut() {
-	// 		monitor.update_chunk_store_fsstats();
-	// 	}
-	// }
-
 	pub fn get_monitor_for_file_path(&mut self, logfile: &String) -> Option<&mut LogMonitor> {
 		let mut monitor_for_path = None;
 		for (monitor_file, monitor) in self.monitors.iter_mut() {
@@ -222,6 +223,8 @@ impl App {
 	}
 
 	pub fn set_logfile_with_focus(&mut self, logfile_name: String) {
+		if logfile_name.len() == 0 { return; }
+
 		match self.get_monitor_with_focus() {
 			Some(fading_monitor) => {
 				fading_monitor.has_focus = false;
@@ -247,6 +250,10 @@ impl App {
 	}
 
 	pub fn change_focus_next(&mut self) {
+		if self.logfiles_manager.logfiles_added.len() == 0 { return; }
+
+		let opt_debug_window = { let opt = OPT.lock().unwrap(); opt.debug_window };
+
 		if self.dash_state.main_view == DashViewMain::DashDebug {
 			return;
 		}
@@ -259,21 +266,21 @@ impl App {
 		}
 
 		let mut next_i = 0;
-		for (i, name) in self.dash_state.logfile_names.iter().enumerate() {
+		for (i, name) in self.logfiles_manager.logfiles_added.iter().enumerate() {
 			if name == &self.logfile_with_focus {
-				if i < self.dash_state.logfile_names.len() - 1 {
+				if i < self.logfiles_manager.logfiles_added.len() - 1 {
 					next_i = i + 1;
 				}
 				break;
 			}
 		}
 
-		if next_i == 0 && self.opt.debug_window && self.logfile_with_focus != DEBUG_WINDOW_NAME {
+		if next_i == 0 && opt_debug_window && self.logfile_with_focus != DEBUG_WINDOW_NAME {
 			self.set_logfile_with_focus(DEBUG_WINDOW_NAME.to_string());
 			return;
 		}
 
-		let logfile = self.dash_state.logfile_names[next_i].to_string();
+		let logfile = self.logfiles_manager.logfiles_added[next_i].to_string();
 		self.set_logfile_with_focus(logfile.clone());
 
 		if let Some(debug_logfile) = self.get_debug_dashboard_logfile() {
@@ -284,6 +291,10 @@ impl App {
 	}
 
 	pub fn change_focus_previous(&mut self) {
+		if self.logfiles_manager.logfiles_added.len() == 0 { return; }
+
+		let opt_debug_window = { let opt = OPT.lock().unwrap(); opt.debug_window };
+
 		if self.dash_state.main_view == DashViewMain::DashDebug {
 			return;
 		}
@@ -295,9 +306,9 @@ impl App {
 			}
 		}
 
-		let len = self.dash_state.logfile_names.len();
+		let len = self.logfiles_manager.logfiles_added.len();
 		let mut previous_i = len - 1;
-		for (i, name) in self.dash_state.logfile_names.iter().enumerate() {
+		for (i, name) in self.logfiles_manager.logfiles_added.iter().enumerate() {
 			if name == &self.logfile_with_focus {
 				if i > 0 {
 					previous_i = i - 1;
@@ -306,7 +317,7 @@ impl App {
 			}
 		}
 
-		if self.opt.debug_window
+		if opt_debug_window
 			&& previous_i == len - 1
 			&& self.logfile_with_focus != DEBUG_WINDOW_NAME
 		{
@@ -314,7 +325,7 @@ impl App {
 			return;
 		}
 
-		let logfile = self.dash_state.logfile_names[previous_i].to_string();
+		let logfile = self.logfiles_manager.logfiles_added[previous_i].to_string();
 		self.set_logfile_with_focus(logfile.clone());
 
 		if let Some(debug_logfile) = self.get_debug_dashboard_logfile() {
@@ -325,8 +336,8 @@ impl App {
 	}
 
 	pub fn change_focus_to(&mut self, logfile_index: usize) {
-		if logfile_index < self.dash_state.logfile_names.len() {
-			self.set_logfile_with_focus(self.dash_state.logfile_names[logfile_index].clone());
+		if logfile_index < self.logfiles_manager.logfiles_added.len() {
+			self.set_logfile_with_focus(self.logfiles_manager.logfiles_added[logfile_index].clone());
 			self.dash_state.main_view = DashViewMain::DashNode;
 		}
 	}
@@ -336,12 +347,16 @@ impl App {
 	pub fn handle_arrow_down(&mut self) { self.handle_arrow( true); }
 
 	pub fn handle_arrow(&mut self, is_down: bool) {
+		if self.logfiles_manager.logfiles_added.len() == 0 { return; }
+
+		let opt_debug_window = { let opt = OPT.lock().unwrap(); opt.debug_window };
+
 		let list = match self.dash_state.main_view {
 			DashViewMain::DashSummary => { Some(&mut self.dash_state.summary_window_rows) }
 			DashViewMain::DashNode => {
 				if let Some(monitor) = self.get_monitor_with_focus() {
 					Some(&mut monitor.content)
-				} else if self.opt.debug_window {
+				} else if opt_debug_window {
 					Some(&mut self.dash_state.debug_window_list)
 				} else {
 					None
@@ -349,7 +364,7 @@ impl App {
 			}
 			DashViewMain::DashHelp => { None }
 			DashViewMain::DashDebug => {
-				if self.opt.debug_window {
+				if opt_debug_window {
 					Some(&mut self.dash_state.debug_window_list)
 				} else {
 					None
@@ -363,10 +378,12 @@ impl App {
 	}
 
 	pub fn preserve_node_selection(&mut self) {
+		if self.logfiles_manager.logfiles_added.len() == 0 { return; }
+
 		if self.dash_state.main_view == DashViewMain::DashSummary {
 			if let Some(selected_index) = self.dash_state.summary_window_rows.state.selected() {
 				let selected_logfile = &self.dash_state.logfile_names_sorted[selected_index];
-				if let Some(node_index) = self.dash_state.logfile_names.iter().position(|s| s == selected_logfile.as_str()) {
+				if let Some(node_index) = self.logfiles_manager.logfiles_added.iter().position(|s| s == selected_logfile.as_str()) {
 					self.change_focus_to(node_index);
 				}
 			}
@@ -387,51 +404,51 @@ impl App {
 		}
 	}
 
-		// TODO this regenerates every line. May be worth just updating the line for the updated node/monitor
-		// Needs to be on the app to manage focus for DashSummary and DashNode through sorting of summary table
-		pub fn update_summary_window(&mut self) {
-			let current_selection = self.dash_state.summary_window_rows.state.selected();
+	// TODO this regenerates every line. May be worth just updating the line for the updated node/monitor
+	// Needs to be on the app to manage focus for DashSummary and DashNode through sorting of summary table
+	pub fn update_summary_window(&mut self) {
+		let current_selection = self.dash_state.summary_window_rows.state.selected();
 
-			self.dash_state.summary_window_rows = StatefulList::new();
+		self.dash_state.summary_window_rows = StatefulList::new();
 
-			// TODO could avoid this repeated copy by ensuring both are modified at the same time
-			self.dash_state.logfile_names_sorted = self.dash_state.logfile_names
-				.iter()
-				.map(|f| f.clone())
-				.collect();
+		// TODO could avoid this repeated copy by ensuring both are modified at the same time
+		self.dash_state.logfile_names_sorted = self.logfiles_manager.logfiles_added
+			.iter()
+			.map(|f| f.clone())
+			.collect();
 
-			crate::custom::ui_summary_table::sort_nodes_by_column(&mut self.dash_state, &mut self.monitors);
+		super::ui_summary_table::sort_nodes_by_column(&mut self.dash_state, &mut self.monitors);
 
-			for i in 0..self.dash_state.logfile_names_sorted.len() {
-				let filepath = self.dash_state.logfile_names_sorted[i].clone();
-				if let Some(monitor) = self.monitors.get_mut(&filepath) {
-					if !monitor.is_debug_dashboard_log {
-						monitor.metrics.update_node_status_string();
-						let node_summary = crate::custom::ui_summary_table::format_table_row(monitor);
-						self.append_to_summary_window(&node_summary);
-					}
+		for i in 0..self.dash_state.logfile_names_sorted.len() {
+			let filepath = self.dash_state.logfile_names_sorted[i].clone();
+			if let Some(monitor) = self.monitors.get_mut(&filepath) {
+				if !monitor.is_debug_dashboard_log {
+					monitor.metrics.update_node_status_string();
+					let node_summary = super::ui_summary_table::format_table_row(monitor);
+					self.append_to_summary_window(&node_summary);
 				}
 			}
-
-			self.dash_state.summary_window_rows.state.select(current_selection);
 		}
 
-		fn append_to_summary_window(&mut self, text: &str){
-			self.dash_state.summary_window_rows.items.push(text.to_string());
+		self.dash_state.summary_window_rows.state.select(current_selection);
+	}
 
-			let len = self.dash_state.summary_window_rows.items.len();
+	fn append_to_summary_window(&mut self, text: &str){
+		self.dash_state.summary_window_rows.items.push(text.to_string());
 
-			if len > self.dash_state.max_summary_window {
-				self.dash_state.summary_window_rows.items = self.dash_state
-					.summary_window_rows
-					.items
-					.split_off(len - self.dash_state.max_summary_window);
-			} else {
-				self.dash_state.summary_window_rows.state.select(Some(len - 1));
-			}
+		let len = self.dash_state.summary_window_rows.items.len();
 
+		if len > self.dash_state.max_summary_window {
+			self.dash_state.summary_window_rows.items = self.dash_state
+				.summary_window_rows
+				.items
+				.split_off(len - self.dash_state.max_summary_window);
+		} else {
+			self.dash_state.summary_window_rows.state.select(Some(len - 1));
 		}
 
+	}
+
 	pub fn toggle_logfile_area(&mut self) {
 		self.dash_state.node_logfile_visible = !self.dash_state.node_logfile_visible;
 	}
@@ -500,15 +517,13 @@ fn do_bracketed_next_previous(list: &mut StatefulList<String>, next: bool) {
 }
 
 fn exit_with_usage(reason: &str) -> Result<App, std::io::Error> {
-	println!(
+	eprintln!(
 		"Try '{} --help' for more information.",
 		Opt::clap().get_name()
 	);
 	return Err(Error::new(ErrorKind::Other, reason));
 }
 
-use fs2::FsStats;
-
 const NODE_INACTIVITY_TIMEOUT_S: i64 = 20;	// Seconds with no log message before node becomes 'inactive'
 
 pub struct LogMonitor {
@@ -517,20 +532,36 @@ pub struct LogMonitor {
 	max_content: usize, // Limit number of lines in content
 	pub has_focus: bool,
 	pub logfile: String,
-	pub chunk_store_fsstats: Option<FsStats>,
-	pub chunk_store_pathbuf: PathBuf,
 	pub metrics: NodeMetrics,
 	pub metrics_status: StatefulList<String>,
 	pub is_debug_dashboard_log: bool,
+	pub latest_checkpoint_time: Option<DateTime<Utc>>,
 }
 
 use std::sync::atomic::{AtomicUsize, Ordering};
 static NEXT_MONITOR: AtomicUsize = AtomicUsize::new(0);
 
-impl LogMonitor {
-	pub fn new(opt: &Opt, logfile_path: String, max_lines: usize) -> LogMonitor {
-		let index = NEXT_MONITOR.fetch_add(1, Ordering::Relaxed);
+fn next_unused_index(monitors: &mut HashMap<String, LogMonitor>) -> usize {
+	let mut next_index = 0;
 
+	let mut index_unused = false;
+	while !index_unused {
+		next_index = NEXT_MONITOR.fetch_add(1, Ordering::Relaxed);
+
+		index_unused = true;
+		for (_logfile, monitor) in monitors.iter()  {
+			if next_index == monitor.index { index_unused = false; }
+		}
+	}
+
+	next_index
+}
+
+
+use super::logfile_checkpoints::LogfileCheckpoint;
+
+impl LogMonitor {
+	pub fn new(logfile_path: String) -> LogMonitor {
 		let mut is_debug_dashboard_log = false;
 		if let Some(debug_logfile) = &*DEBUG_LOGFILE.lock().unwrap() {
 			if let Some(debug_logfile_path) = debug_logfile.path().to_str() {
@@ -538,35 +569,91 @@ impl LogMonitor {
 			}
 		}
 
-		let mut chunk_store_pathbuf = PathBuf::from(&logfile_path);
-		if chunk_store_pathbuf.pop() {
-			chunk_store_pathbuf.push("chunkdb")
-		}
-
+		let opt_lines_max = { OPT.lock().unwrap().lines_max };
 		LogMonitor {
-			index,
+			index: 0,
 			logfile: logfile_path,
-			max_content: max_lines,
-			chunk_store_fsstats: None,
-			chunk_store_pathbuf,
-			metrics: NodeMetrics::new(&opt),
+			max_content: opt_lines_max,
+			metrics: NodeMetrics::new(),
 			content: StatefulList::with_items(vec![]),
 			has_focus: false,
 			metrics_status: StatefulList::with_items(vec![]),
 			is_debug_dashboard_log,
+			latest_checkpoint_time: None,
+		}
+	}
+
+	/// Resolve any clash between self.index and index of other monitors which may happen
+	/// when mixing creation of new monitors with initialisation by restoring a checkpoint.
+	///
+	/// For a restored checkpoint the metrics should be set, so if one has metrics and the other
+	/// doesn't, the former is treated as older and given the lower index.
+	pub fn canonicalise_monitor_index(&mut self, monitors: &mut HashMap<String, LogMonitor>) {
+		if self.index == 0 { self.index = NEXT_MONITOR.fetch_add(1, Ordering::Relaxed); }
+
+		let existing_index = NEXT_MONITOR.fetch_add(0, Ordering::Relaxed);
+		let next_index = next_unused_index(monitors);
+
+		let mut clash_monitor = None;
+		for (other_logfile, other) in monitors.iter_mut() {
+			if self.index == other.index && &self.logfile != other_logfile {
+				clash_monitor = Some(other);
+			}
+		}
+
+		if let Some(other) = clash_monitor {
+
+			let mut lower_index = self.index;
+			let mut higher_index = next_index;
+			if lower_index > higher_index {
+				lower_index = next_index;
+				higher_index = self.index;
+			}
+
+			// Default
+			self.index = higher_index;
+			other.index = lower_index;
+
+			// If we know the earlier of the two metrics, use that to order the index in self and other
+			if let Some(self_start_time) = self.metrics.node_started {
+				let flip = if let Some(other_start_time) = other.metrics.node_started {
+					self_start_time < other_start_time
+				} else {
+					true
+				};
+				if flip {
+					self.index = lower_index;
+					other.index = higher_index;
+				}
+			}
+		} else {
+			// next_index not used so restore state to avoid unnecessary increments
+			NEXT_MONITOR.store(existing_index, Ordering::Relaxed);
 		}
 	}
 
 	pub fn is_node(&self) -> bool { return !self.is_debug_dashboard_log; }
 
-	// pub fn update_chunk_store_fsstats(&mut self) {
-	// 	self.chunk_store_fsstats = match statvfs(&self.chunk_store_pathbuf) {
-	// 		Ok(fsstats) => Some(fsstats),
-	// 		Err(_) => None,
-	// 	};
-	// }
+	pub fn from_checkpoint(&mut self, checkpoint: &LogfileCheckpoint) {
+		self.index = checkpoint.monitor_index;
+		self.latest_checkpoint_time = checkpoint.latest_entry_time;
+		self.metrics = checkpoint.monitor_metrics.clone();
+	}
+
+	pub fn to_checkpoint(&mut self, checkpoint: &mut LogfileCheckpoint) {
+		checkpoint.latest_entry_time = self.latest_checkpoint_time;
+		checkpoint.monitor_index = self.index;
+		checkpoint.monitor_metrics = self.metrics.clone();
+	}
+
+	// TODO if speed is an issue look at speeding up:
+	// TODO - LogEntry::decode_metadata()
+	// TODO - finding first log entry to decode using a bisection search
+	pub fn load_logfile_from_time(&mut self, dash_state: &mut DashState, after_time: Option<DateTime<Utc>>) -> std::io::Result<()> {
+		if let Some(after_time) = after_time {
+			dash_state.vdash_status.message(&format!("loading logfile after time: {}", after_time).to_string(), None);
+		}
 
-	pub fn load_logfile(&mut self, dash_state: &mut DashState) -> std::io::Result<()> {
 		use std::io::{BufRead, BufReader};
 
 		let f = File::open(self.logfile.to_string());
@@ -575,12 +662,11 @@ impl LogMonitor {
 			Err(_e) => return Ok(()), // It's ok for a logfile not to exist yet
 		};
 
-		// self.update_chunk_store_fsstats();
 		let f = BufReader::new(f);
 
 		for line in f.lines() {
 			let line = line.expect("Unable to read line");
-			self.append_to_content(&line)?;
+			self.append_to_content_from_time(dash_state, &line, after_time)?;
 			if self.is_debug_dashboard_log {
 				dash_state._debug_window(&line);
 			}
@@ -595,14 +681,69 @@ impl LogMonitor {
 		Ok(())
 	}
 
-	pub fn append_to_content(&mut self, text: &str) -> Result<(), std::io::Error> {
-		if self.line_filter(&text) {
-			self._append_to_content(text)?; // Show in TUI
-			if self.is_debug_dashboard_log {
-				return Ok(());
-			}
-			self.metrics.gather_metrics(&text)?;
+	pub fn append_to_content(&mut self, line: &str, checkpoint_interval: u64) -> Result<String, std::io::Error> {
+		self.metrics.parser_output = format!("LogMeta::decode_metadata() failed on: {}", line); // For debugging
+		// debug_log!(&self.parser_output.clone());
+
+		self.metrics.entry_metadata = LogEntry::decode_metadata(line);
+
+		if self.metrics.entry_metadata.is_none() {
+			// debug_log!("gather_metrics() - skipping bec. metadata missing");
+			return Ok("".to_string());	// Skip until start of first log message
 		}
+
+		self._append_to_content(line)?; // Show in TUI
+		if self.is_debug_dashboard_log {
+			return Ok("".to_string());
+		}
+
+		self.metrics.gather_metrics(&line)?;
+
+		if checkpoint_interval > 0 { 	// Checkpoints disabled by zero interval
+			return self.update_checkpoint(checkpoint_interval);
+		}
+
+		Ok("".to_string())
+	}
+
+	pub fn update_checkpoint(&mut self, checkpoint_interval: u64) -> Result<String, Error> {
+		if let Some(metadata) = &self.metrics.entry_metadata {
+			if self.latest_checkpoint_time.is_none() {
+				return save_checkpoint(self);
+			} else {
+				if let Some(latest_checkpoint_time) = self.latest_checkpoint_time {
+					if latest_checkpoint_time + Duration::seconds(checkpoint_interval as i64) < metadata.message_time {
+						return save_checkpoint(self);
+					}
+				}
+			}
+		}
+
+		Ok("".to_string())
+	}
+
+	pub fn append_to_content_from_time(&mut self, _dash_state: &mut DashState, line: &str, after_time: Option<DateTime<Utc>>) -> Result<(), std::io::Error> {
+		self.metrics.parser_output = format!("LogMeta::decode_metadata() failed on: {}", line); // For debugging
+		// debug_log!(&self.parser_output.clone());
+
+		if let Some(entry_metadata) = LogEntry::decode_metadata(line) {
+			if let Some(after_time) = after_time {
+				if !entry_metadata.message_time.gt(&after_time) { return Ok(()); }
+			}
+
+			self.metrics.entry_metadata = Some(entry_metadata);
+		} else {
+			// debug_log!("gather_metrics() - skipping bec. metadata missing");
+			if after_time.is_some() { return Ok(()); }
+		}
+
+		self._append_to_content(line)?; // Show in TUI
+		if self.is_debug_dashboard_log {
+			return Ok(());
+		}
+
+		self.metrics.gather_metrics(&line)?;
+
 		Ok(())
 	}
 
@@ -616,12 +757,6 @@ impl LogMonitor {
 		}
 		Ok(())
 	}
-
-	// Some logfile lines are too numerous to include so we ignore them
-	// Returns true if the line is to be processed
-	fn line_filter(&mut self, _line: &str) -> bool {
-		true
-	}
 }
 
 use regex::Regex;
@@ -631,10 +766,12 @@ lazy_static::lazy_static! {
 }
 
 #[derive(PartialEq)]
+#[derive(Clone, Default, Debug, Serialize, Deserialize)]
 pub enum NodeStatus {
 	Started,
 	Connecting,
 	Connected,
+	#[default]
 	Stopped,
 }
 
@@ -647,6 +784,7 @@ pub fn node_status_as_string(node_status: &NodeStatus) -> String {
 	}
 }
 
+#[derive(Clone, Debug, Serialize, Deserialize)]
 pub struct MmmStat {
 	sample_count:	u64,
 
@@ -682,6 +820,7 @@ impl MmmStat {
 	}
 }
 
+#[derive(Clone, Debug, Serialize, Deserialize)]
 pub struct NodeMetrics {
 	pub node_started: Option<DateTime<Utc>>,
 	pub running_message: Option<String>,
@@ -689,7 +828,6 @@ pub struct NodeMetrics {
 	pub node_process_id: Option<u64>,
 	pub node_peer_id: Option<String>,
 	pub category_count: HashMap<String, usize>,
-	pub activity_history: Vec<ActivityEntry>,
 
 	pub app_timelines: AppTimelines,
 
@@ -727,12 +865,11 @@ pub struct NodeMetrics {
 	pub total_mb_read: f32,
 	pub total_mb_written: f32,
 
-	pub debug_logfile: Option<NamedTempFile>,
-	parser_output: String,
+	pub parser_output: String,
 }
 
 impl NodeMetrics {
-	fn new(opt: &Opt) -> NodeMetrics {
+	pub fn new() -> NodeMetrics {
 		let mut metrics = NodeMetrics {
 			// Start
 			node_started: None,
@@ -742,11 +879,10 @@ impl NodeMetrics {
 			node_peer_id: None,
 
 			// Logfile entries
-			activity_history: Vec::<ActivityEntry>::new(),
 			entry_metadata: None,
 
 			// A predefined set of Timelines (Sparklines)
-			app_timelines: AppTimelines::new(opt),
+			app_timelines: AppTimelines::new(),
 
 			// Counts
 			category_count: HashMap::new(),
@@ -791,7 +927,6 @@ impl NodeMetrics {
 			total_mb_written: 0.0,
 
 			// Debug
-			debug_logfile: None,
 			parser_output: String::from("-"),
 		};
 		metrics.update_timelines(&Utc::now());
@@ -831,18 +966,6 @@ impl NodeMetrics {
 	///! Process a line from a SAFE Node logfile.
 	///! Use a created LogMeta to update metrics.
 	pub fn gather_metrics(&mut self, line: &str) -> Result<(), std::io::Error> {
-		self.parser_output = format!("LogMeta::decode_metadata() failed on: {}", line); // For debugging
-		// debug_log!(&self.parser_output.clone());
-
-		if let Some(metadata) = LogEntry::decode_metadata(line) {
-			self.entry_metadata = Some(metadata);
-		}
-
-		if self.entry_metadata.is_none() {
-			// debug_log!("gather_metrics() - skipping bec. metadata missing");
-			return Ok(());	// Skip until start of first log message
-		}
-
 		let entry = LogEntry { logstring: String::from(line) };
 		let entry_metadata = self.entry_metadata.as_ref().unwrap().clone();
 		let entry_time = entry_metadata.message_time;
@@ -865,7 +988,7 @@ impl NodeMetrics {
 	}
 
 	///! Return a LogMeta and capture metadata for logfile node start:
-	///!	'Running sn_node v0.74.4'
+	///!	'Running safenode v0.98.32'
 	pub fn parse_start(&mut self, line: &String, entry_metadata: &LogMeta) -> bool {
 		let running_prefix = String::from("Running safenode ");
 
@@ -922,7 +1045,7 @@ impl NodeMetrics {
 			self.count_get(&entry_time);
 			self.node_status = NodeStatus::Connected;
 			return true;
-		} else if line.contains("Wrote record") {
+		} else if line.contains("Wrote record") || line.contains("ValidSpendRecordPutFromNetwork") {
 			self.count_put(&entry_time);
 			self.node_status = NodeStatus::Connected;
 			return true;
@@ -937,14 +1060,14 @@ impl NodeMetrics {
 			};
 			return true;
 		} else if line.contains("nanos accepted for record") {
-			if 	let Some(storage_payment) = self.parse_u64("Payment of NanoTokens(", line) {
+			if 	let Some(storage_payment) = self.parse_u64("payment of NanoTokens(", line) {
 				self.count_storage_payment(entry_time, storage_payment);
 				self.parser_output = format!("Payment received: {}", storage_payment);
 				return true;
 			};
-		} else if line.contains("connected peers") {
+		} else if line.contains("PeersInRoutingTable") {
 			let mut parser_output = String::from("connected peers:");
-			if let Some(peers_connected) = self.parse_u64("now we have #", line) {
+			if let Some(peers_connected) = self.parse_u64("PeersInRoutingTable(", line) {
 				self.count_peers_connected(entry_time, peers_connected);
 				parser_output = format!("{} {}", &parser_output, peers_connected);
 			};
@@ -965,8 +1088,6 @@ impl NodeMetrics {
 				response = line.as_str()[response_start..response_start + response_end]
 					.as_ref();
 				if !response.is_empty() {
-					let activity_entry = ActivityEntry::new(&line, &self.entry_metadata.as_ref().unwrap(), response);
-					self.activity_history.push(activity_entry);
 					self.parser_output = format!("node activity: {}", response);
 				}
 			}
@@ -1233,36 +1354,8 @@ impl NodeMetrics {
 	}
 }
 
-///! Node activity for node activity_history
-pub struct ActivityEntry {
-	pub message: String,
-	pub activity: String,
-	pub logstring: String,
-	pub category: String, // First word, "Running", "INFO", "WARN" etc
-	pub message_time: DateTime<Utc>,
-	pub source: String,
-
-	pub parser_output: String,
-}
-
-impl ActivityEntry {
-	pub fn new(line: &String, entry_metadata: &LogMeta, activity: &str) -> ActivityEntry {
-		ActivityEntry {
-			message: entry_metadata.message.clone(),
-			activity: activity.to_string(),
-			logstring: line.clone(),
-			category: entry_metadata.category.clone(),
-			message_time: entry_metadata.message_time,
-			source: entry_metadata.source.clone(),
-
-			parser_output: String::from(""),
-		}
-	}
-}
-
-
 ///! Metadata for a logfile line
-#[derive(Clone)]
+#[derive(Clone, Debug, Serialize, Deserialize)]
 pub struct LogMeta {
 	pub category: String, // First word ('INFO', 'WARN' etc.)
 	pub message_time: DateTime<Utc>,
@@ -1285,6 +1378,7 @@ impl LogMeta {
 		}
 	}
 }
+
 ///! Used to build a history of what is in the log, one LogMeta per line
 pub struct LogEntry {
 	pub logstring: String,			// One line of raw text from the logfile
@@ -1339,6 +1433,8 @@ impl LogEntry {
 
 ///! Active UI at top level
 #[derive(PartialEq)]
+#[derive(Clone)]
+#[derive(Copy)]
 pub enum DashViewMain {
 	DashSummary,
 	DashNode,
@@ -1347,8 +1443,9 @@ pub enum DashViewMain {
 }
 
 pub struct DashState {
+	pub vdash_status: StatusMessage,
 	pub main_view: DashViewMain,
-	pub logfile_names: Vec<String>,
+	pub previous_main_view: DashViewMain,
 	pub logfile_names_sorted: Vec<String>,
 	pub logfile_names_sorted_ascending: bool,
 
@@ -1373,12 +1470,18 @@ pub struct DashState {
 	max_debug_window: usize,
 }
 
+const UI_STATUS_DEFAULT_MESSAGE: &str = "Press '?' for Help";
+const UI_STATUS_DEFAULT_DURATION_S: i64 = 5;
+use super::ui_status::StatusMessage;
+
 impl DashState {
 	pub fn new() -> DashState {
 
 		let mut new_dash = DashState {
+			vdash_status: StatusMessage::new(&String::from(UI_STATUS_DEFAULT_MESSAGE), &Duration::seconds(UI_STATUS_DEFAULT_DURATION_S)),
+
 			main_view: DashViewMain::DashSummary,
-			logfile_names: Vec::<String>::new(),				// Ordered in sync with LogMonitor.index?
+			previous_main_view: DashViewMain::DashSummary,
 			logfile_names_sorted: Vec::<String>::new(),	// Sorted by column
 			logfile_names_sorted_ascending: true,
 
@@ -1401,7 +1504,7 @@ impl DashState {
 			debug_window_list: StatefulList::new(),
 			max_debug_window: 100,
 		};
-		crate::custom::ui_summary_table::initialise_summary_headings(&mut new_dash);
+		super::ui_summary_table::initialise_summary_headings(&mut new_dash);
 		new_dash
 	}
 
@@ -1458,6 +1561,7 @@ pub fn set_main_view(view: DashViewMain, app: &mut App) {
 		return;
 	}
 
+	app.dash_state.previous_main_view = app.dash_state.main_view;
 	save_focus(app);
 	app.dash_state.main_view = view;
 	restore_focus(app);
