@@ -1,10 +1,13 @@
 pub mod app;
 pub mod app_timelines;
-pub mod timelines;
+pub mod logfile_checkpoints;
+pub mod logfiles_manager;
 pub mod opt;
+pub mod timelines;
 pub mod ui;
 pub mod ui_debug;
 pub mod ui_help;
 pub mod ui_node;
 pub mod ui_summary_table;
 pub mod ui_summary;
+pub mod ui_status;
