@@ -25,17 +25,29 @@ pub struct Opt {
 	#[structopt(short, long)]
 	pub ignore_existing: bool,
 
+	/// A *nix 'glob' path to match multiple files.
+	/// Can be provided multiple times as here:
+	///
+	///   vdash -g "$HOME/.local/share/safe/node/**/safenode.log" -g "./remote-node-logs/*/logs/safenode.log"
+	#[structopt(name="glob-path", short, long, multiple=true)]
+	pub glob_paths: Vec<String>,
+
+	/// Enable periodic scan of any glob paths every so many seconds. 0 to disable.
+	#[structopt(long, default_value = "0")]
+	pub glob_scan: i64,
+
+	/// Set checkpoint interval in seconds. 0 to disable checkpoints.
+	#[structopt(long, default_value = "300")]
+	pub checkpoint_interval: u64,
+
 	/// One or more logfiles to monitor
 	#[structopt(name = "LOGFILE")]
 	pub files: Vec<String>,
 
-	/// Show a debug window to the right of the logfile view in main dashboard
+	/// Parses first logfile *only* and adds a debug output window (accessed with l/r arrow)
+	/// Also shows smaller debug output window to the right of the node view for the logfile
 	#[structopt(short, long)]
 	pub debug_window: bool,
-
-	/// Parses first logfile and adds a debug output window (fullscreen with r/l arrow))
-	#[structopt(long)]
-	pub debug_dashboard: bool,
 }
 
 pub fn get_app_name() -> String { String::from(Opt::clap().get_name()) }
